import random

def tobin(n):
    return bin(n).replace("0b", "")

def check_default(our_answer, their_answer):
    return our_answer.lower() == their_answer.lower()

def task_identity():
    n = random.randint(1, 50)
    s = tobin(n)
    return { "var1": s, "answer": s }

def task_substr():
    n = random.randint(2**25, 2**28)
    s = tobin(n)
    return { "var1": s, "answer": s[0:6] }

def task_reverse():
    n = random.randint(5, 50) * 2 + 1
    s = tobin(n)
    return { "var1": s, "answer": s[::-1] }

def task_not():
    n = random.randint(1, 8)
    s = tobin(n)
    return { "var1": s, "answer": s }

def check_not(our_answer, their_answer):
    return their_answer in ("1", "10", "11", "100", "101", "110", "111", "1000") and their_answer != our_answer

def task_eval():
    while True:
        n = random.randint(1, 9)
        s = str(n)
        for i in range(random.randint(1, 4)):
            op = random.randrange(5)
            arg = random.randint(1, 9)
            if op == 1: n, s = n + arg, "({} + {})".format(s, arg)
            if op == 2: n, s = n - arg, "({} - {})".format(s, arg)
            if op == 3: n, s = n * arg, "({} * {})".format(s, arg)
            if op == 4: n, s = arg + n, "({} + {})".format(arg, s)
            if op == 5: n, s = arg - n, "({} - {})".format(arg, s)
            if op == 6: n, s = arg * n, "({} * {})".format(arg, s)
        if n > 0 and n < 100: break
    return { "var1": s, "answer": tobin(n) }

def task_sqrt():
    n = random.randint(1, 20)
    return { "var1": tobin(n*n), "answer": tobin(n) }

def task_factorial():
    n = random.randint(2, 10)
    f = 1
    for i in range(1, n+1): f *= i
    return { "var1": tobin(n) + "!", "answer": tobin(f) }

def task_parentheses():
    s = ""
    for _ in range(4):
        i = random.randint(0, len(s))
        x = random.choice(["[]", "()"])
        s = s[:i] + x + s[i:]
    if random.randint(0, 1):
        ans = "YES"
    else:
        ans = "NO"
        i = random.randint(0, len(s) - 1)
        def change(x):
            while True:
                y = random.choice("[]()")
                if x != y:
                    return y
        s = s[:i] + change(s[i]) + s[i+1:]

    return { "var1": s, "answer": ans }

def int_to_roman(input):
    """Convert an integer to Roman numerals.

    http://code.activestate.com/recipes/81611-roman-numerals/
    """
    ints = (1000, 900,  500, 400,  100,  90,  50,   40,  10,   9,    5,   4,    1)
    nums = ('M',  'CM', 'D', 'CD', 'C', 'XC', 'L', 'XL', 'X', 'IX', 'V', 'IV', 'I')
    result = ""
    for i in range(len(ints)):
        count = int(input / ints[i])
        result += nums[i] * count
        input -= ints[i] * count
    return result

def task_roman():
    m = random.choice([15, 150, 150, 1500])
    x = random.randint(1, m)
    y = random.randint(1, m)
    return { "var1": int_to_roman(x), "var2": int_to_roman(y), "answer": int_to_roman(x + y) }

if __name__ == "__main__":
    from subtask_tictac import task_tictactoe
    from subtask_hexominoes import task_hexominoes
    from subtask_components import task_components
    from subtask_ipschuman import task_ipschuman
else:
    from .subtask_tictac import task_tictactoe
    from .subtask_hexominoes import task_hexominoes
    from .subtask_components import task_components
    from .subtask_ipschuman import task_ipschuman

texts = {
    "identity": "HELLO I WOULD LIKE TO ORDER FOOD ITEM var1.",
    "substr":
        "GOOD DAY COUGH GIVE ME FOOD var1 COUGH COUGH COUGH. "
        "PARDON ME, THAT WAS A BUFFER OVERFLOW. I WANTED TO SAY THE "
        "FIRST 110 DIGITS OF THAT, PLEASE IGNORE THE REST.",
    "reverse": "OLLEH, I MA A BOTOR TAHT SKLAT SDRAWKCAB. EVIG EM DOOF var1 ESAELP.",
    "not": "GIVE ME ANY FOOD BETWEEN 1 AND 1000, INCLUSIVE. BUT NOT var1, I HATE THAT.",
    "eval":
        "I HEARD var1 IS VERY TASTY, BUT I DO NOT UNDERSTAND THAT FOREIGN LANGUAGE. "
        "WHAT IS IT CALLED IN BINARY?",
    "sqrt": "I WANT THE SQUARE ROOT OF var1.",
    "factorial": "I AM SO EXCITED! GIVE ME var1 AND YES, OF COURSE THAT IS A FACTORIAL!",
    "parentheses": "I JUST WANT TO KNOW: IS var1 CORRECTLY NESTED? PLEASE ANSWER YES OR NO.",
    "hexominoes": "CAN YOU CHANGE THIS SHAPE TO THAT SHAPE JUST BY ROTATING OR FLIPPING IT? ANSWER YES OR NO.\n\nvar1\n\n\n\nvar2",
    "tictactoe": "A STRANGE GAME. THE ONLY WINNING MOVE IS NOT TO PLAY.\nBUT WHO WON? ANSWER X OR O OR NEITHER.\n\nvar1",
    "components": "HOW MANY ISLANDS ARE ON THIS MAP?\n\nvar1",
    "roman": "Ave, popina dominus! Lorem ipsum! I heard you have Roman recipes too. Give me food var1 + var2.",
    "ipschuman": "Hi there. I'm a huge fan of IPSC history! Please give me the answer to this question:\nvar1\nPlease give me the food in decimal. I'm binary-intolerant.",
}

tasks = {
    "identity": task_identity,
    "substr": task_substr,
    "reverse": task_reverse,
    "not": task_not,
    "eval": task_eval,
    "sqrt": task_sqrt,
    "factorial": task_factorial,
    "parentheses": task_parentheses,
    "hexominoes": task_hexominoes,
    "tictactoe": task_tictactoe,
    "components": task_components,
    "roman": task_roman,
    "ipschuman": task_ipschuman,
}

checkers = {
    "not": check_not,
}

if __name__ == "__main__":
    import timeit
    N=10000
    print("identity [ms]", timeit.timeit('task_identity()', 'from __main__ import task_identity', number=N)*1000/N)
    print("hexominoes [ms]", timeit.timeit('task_hexominoes()', 'from __main__ import task_hexominoes', number=N)*1000/N)
    print("parentheses [ms]", timeit.timeit('task_parentheses()', 'from __main__ import task_parentheses', number=N)*1000/N)
    print("tictactoe [ms]", timeit.timeit('task_tictactoe()', 'from __main__ import task_tictactoe', number=N)*1000/N)
    print("components [ms]", timeit.timeit('task_components()', 'from __main__ import task_components', number=N)*1000/N)
